// Copyright 2016 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

// Generated from template:
//   core/css/properties/templates/css_property_subclass.h.tmpl
// and input files:
//   ../../3rdparty/chromium/third_party/blink/renderer/core/css/ComputedStyleFieldAliases.json5
//   ../../3rdparty/chromium/third_party/blink/renderer/core/css/CSSProperties.json5
//   ../../3rdparty/chromium/third_party/blink/renderer/core/css/properties/CSSPropertyMethods.json5


#ifndef THIRD_PARTY_BLINK_RENDERER_CORE_CSS_PROPERTIES_LONGHAND_BORDER_IMAGE_REPEAT_H_
#define THIRD_PARTY_BLINK_RENDERER_CORE_CSS_PROPERTIES_LONGHAND_BORDER_IMAGE_REPEAT_H_

#include "third_party/blink/renderer/core/css/css_identifier_value.h"
#include "third_party/blink/renderer/core/css/css_primitive_value_mappings.h"
#include "third_party/blink/renderer/core/css/properties/longhand.h"
#include "third_party/blink/renderer/core/css/resolver/style_resolver_state.h"
#include "third_party/blink/renderer/core/style/computed_style.h"

namespace blink {
namespace CSSLonghand {

// Implements the 'border-image-repeat' CSS property
// See src/third_party/WebKit/Source/core/css/properties/README.md
class BorderImageRepeat final : public Longhand {
 public:
  constexpr BorderImageRepeat() : Longhand() {}
  const char* GetPropertyName() const override { return "border-image-repeat\0"; }
  const WTF::AtomicString& GetPropertyNameAtomicString() const override {
    static const WTF::AtomicString& name = WTF::AtomicString("border-image-repeat\0");
    return name;
  }
  const WTF::String GetJSPropertyName() const override {
    return WTF::String("borderImageRepeat\0");
  }
  CSSPropertyID PropertyID() const override { return CSSPropertyBorderImageRepeat; }
  const CSSValue* ParseSingleValue(CSSParserTokenRange&, const CSSParserContext&, const CSSParserLocalContext&) const override;
  const CSSValue* CSSValueFromComputedStyleInternal(const ComputedStyle&, const SVGComputedStyle&, const LayoutObject*, Node*, bool allow_visited_style) const override;
  const CSSValue* InitialValue() const override;

  // Style builder functions
  void ApplyInitial(StyleResolverState& state) const override {
    const NinePieceImage& current_image = state.Style()->BorderImage();
    if (current_image.HorizontalRule() == kStretchImageRule &&
        current_image.VerticalRule() == kStretchImageRule)
      return;
    NinePieceImage image(current_image);
    image.SetHorizontalRule(kStretchImageRule);
    image.SetVerticalRule(kStretchImageRule);
    state.Style()->SetBorderImage(image);
  }
  void ApplyInherit(StyleResolverState& state) const override {
    NinePieceImage image(state.Style()->BorderImage());
    image.CopyRepeatFrom(state.ParentStyle()->BorderImage());
    state.Style()->SetBorderImage(image);
  }
  void ApplyValue(StyleResolverState& state, const CSSValue& value) const override {
    NinePieceImage image(state.Style()->BorderImage());
    CSSToStyleMap::MapNinePieceImageRepeat(state, value, image);
    state.Style()->SetBorderImage(image);
  }
};

}  // namespace CSSLonghand
}  // namespace blink

#endif  // THIRD_PARTY_BLINK_RENDERER_CORE_CSS_PROPERTIES_LONGHAND_BORDER_IMAGE_REPEAT_H_
