// Copyright 2016 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

// Generated from template:
//   core/css/properties/templates/css_property_subclass.h.tmpl
// and input files:
//   ../../3rdparty/chromium/third_party/blink/renderer/core/css/ComputedStyleFieldAliases.json5
//   ../../3rdparty/chromium/third_party/blink/renderer/core/css/CSSProperties.json5
//   ../../3rdparty/chromium/third_party/blink/renderer/core/css/properties/CSSPropertyMethods.json5


#ifndef THIRD_PARTY_BLINK_RENDERER_CORE_CSS_PROPERTIES_LONGHAND_MIN_BLOCK_SIZE_H_
#define THIRD_PARTY_BLINK_RENDERER_CORE_CSS_PROPERTIES_LONGHAND_MIN_BLOCK_SIZE_H_

#include "third_party/blink/renderer/core/css/css_identifier_value.h"
#include "third_party/blink/renderer/core/css/css_primitive_value_mappings.h"
#include "third_party/blink/renderer/core/css/properties/longhand.h"
#include "third_party/blink/renderer/core/css/resolver/style_resolver_state.h"
#include "third_party/blink/renderer/core/style/computed_style.h"
#include "third_party/blink/renderer/core/style_property_shorthand.h"

namespace blink {
namespace CSSLonghand {

// Implements the 'min-block-size' CSS property
// See src/third_party/WebKit/Source/core/css/properties/README.md
class MinBlockSize final : public Longhand {
 public:
  constexpr MinBlockSize() : Longhand() {}
  const char* GetPropertyName() const override { return "min-block-size\0"; }
  const WTF::AtomicString& GetPropertyNameAtomicString() const override {
    static const WTF::AtomicString& name = WTF::AtomicString("min-block-size\0");
    return name;
  }
  const WTF::String GetJSPropertyName() const override {
    return WTF::String("minBlockSize\0");
  }
  CSSPropertyID PropertyID() const override { return CSSPropertyMinBlockSize; }
  const CSSValue* ParseSingleValue(CSSParserTokenRange&, const CSSParserContext&, const CSSParserLocalContext&) const override;
  const CSSValue* CSSValueFromComputedStyleInternal(
      const ComputedStyle& ,
      const SVGComputedStyle&,
      const LayoutObject*,
      Node*,
      bool allow_visited_style) const override {
    // Directional properties are resolved by resolveDirectionAwareProperty()
    // before calling CSSValueFromComputedStyleInternal.
    NOTREACHED();
    return nullptr;
  }
  const CSSProperty& ResolveDirectionAwareProperty(
      TextDirection direction,
      blink::WritingMode writing_mode) const override {
    if (IsHorizontalWritingMode(writing_mode))
      return GetCSSPropertyMinHeight();
    return GetCSSPropertyMinWidth();
  }

  // Style builder functions
  void ApplyInitial(StyleResolverState& state) const override {
    const CSSProperty& resolved_property =
        ResolveDirectionAwareProperty(state.Style()->Direction(),
                                      state.Style()->GetWritingMode());
    DCHECK(!resolved_property.IDEquals(PropertyID()));
    ToLonghand(resolved_property).ApplyInitial(state);
  }
  void ApplyInherit(StyleResolverState& state) const override {
    const CSSProperty& resolved_property =
        ResolveDirectionAwareProperty(state.Style()->Direction(),
                                      state.Style()->GetWritingMode());
    DCHECK(!resolved_property.IDEquals(PropertyID()));
    ToLonghand(resolved_property).ApplyInherit(state);
  }
  void ApplyValue(StyleResolverState& state, const CSSValue& value) const override {
    const CSSProperty& resolved_property =
        ResolveDirectionAwareProperty(state.Style()->Direction(),
                                      state.Style()->GetWritingMode());
    DCHECK(!resolved_property.IDEquals(PropertyID()));
    ToLonghand(resolved_property).ApplyValue(state, value);
  }
};

}  // namespace CSSLonghand
}  // namespace blink

#endif  // THIRD_PARTY_BLINK_RENDERER_CORE_CSS_PROPERTIES_LONGHAND_MIN_BLOCK_SIZE_H_
