// Copyright (c) 2014 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

// Generated from template:
//   templates/origin_trials.cc.tmpl
// and input files:
//   ../../3rdparty/chromium/third_party/blink/renderer/platform/runtime_enabled_features.json5


#include "third_party/blink/renderer/core/origin_trials/origin_trials.h"

#include "third_party/blink/renderer/core/origin_trials/origin_trial_context.h"
#include "third_party/blink/renderer/platform/runtime_enabled_features.h"

namespace blink {

const char OriginTrials::kAnimationWorkletTrialName[] = "AnimationWorklet";
const char OriginTrials::kBudgetQueryTrialName[] = "BudgetQuery";
const char OriginTrials::kCookieStoreTrialName[] = "CookieStore";
const char OriginTrials::kDisableHardwareNoiseSuppressionTrialName[] = "DisableHardwareNoiseSuppression";
const char OriginTrials::kEncryptedMediaHdcpPolicyCheckTrialName[] = "EncryptedMediaHdcpPolicyCheck";
const char OriginTrials::kEventTimingTrialName[] = "EventTiming";
const char OriginTrials::kExperimentalHardwareEchoCancellationTrialName[] = "ExperimentalHardwareEchoCancellation2";
const char OriginTrials::kFeaturePolicyJavaScriptInterfaceTrialName[] = "FeaturePolicyJSAPI";
const char OriginTrials::kInstalledAppTrialName[] = "InstalledApp";
const char OriginTrials::kLowLatencyCanvasTrialName[] = "LowLatencyCanvas";
const char OriginTrials::kMediaCapabilitiesTrialName[] = "MediaCapabilities";
const char OriginTrials::kOriginTrialsSampleAPITrialName[] = "Frobulate";
const char OriginTrials::kOriginTrialsSampleAPIImpliedTrialName[] = "FrobulateImplied";
const char OriginTrials::kRtcPeerConnectionIdTrialName[] = "RtcPeerConnectionId";
const char OriginTrials::kSignatureBasedIntegrityTrialName[] = "SignatureBasedIntegrity";
const char OriginTrials::kStaleWhileRevalidateTrialName[] = "StaleWhileRevalidate";
const char OriginTrials::kTouchEventFeatureDetectionTrialName[] = "ForceTouchEventFeatureDetectionForInspector";
const char OriginTrials::kWebXRTrialName[] = "WebXRDeviceM69";
const char OriginTrials::kWebXRGamepadSupportTrialName[] = "WebXRGamepadSupport";


bool OriginTrials::AnimationWorkletEnabled(const ExecutionContext* executionContext) {
  if (RuntimeEnabledFeatures::AnimationWorkletEnabledByRuntimeFlag())
    return true;

  const OriginTrialContext* context = OriginTrialContext::From(executionContext);
  if (!context) return false;
  if (context->IsTrialEnabled(kAnimationWorkletTrialName))
    return true;
  return false;
}


bool OriginTrials::BudgetQueryEnabled(const ExecutionContext* executionContext) {
  if (RuntimeEnabledFeatures::BudgetQueryEnabledByRuntimeFlag())
    return true;

  const OriginTrialContext* context = OriginTrialContext::From(executionContext);
  if (!context) return false;
  if (context->IsTrialEnabled(kBudgetQueryTrialName))
    return true;
  return false;
}


bool OriginTrials::CookieStoreEnabled(const ExecutionContext* executionContext) {
  if (RuntimeEnabledFeatures::CookieStoreEnabledByRuntimeFlag())
    return true;

  const OriginTrialContext* context = OriginTrialContext::From(executionContext);
  if (!context) return false;
  if (context->IsTrialEnabled(kCookieStoreTrialName))
    return true;
  return false;
}


bool OriginTrials::DisableHardwareNoiseSuppressionEnabled(const ExecutionContext* executionContext) {
  if (RuntimeEnabledFeatures::DisableHardwareNoiseSuppressionEnabledByRuntimeFlag())
    return true;

  const OriginTrialContext* context = OriginTrialContext::From(executionContext);
  if (!context) return false;
  if (context->IsTrialEnabled(kDisableHardwareNoiseSuppressionTrialName))
    return true;
  return false;
}


bool OriginTrials::EncryptedMediaHdcpPolicyCheckEnabled(const ExecutionContext* executionContext) {
  if (RuntimeEnabledFeatures::EncryptedMediaHdcpPolicyCheckEnabledByRuntimeFlag())
    return true;

  const OriginTrialContext* context = OriginTrialContext::From(executionContext);
  if (!context) return false;
  if (context->IsTrialEnabled(kEncryptedMediaHdcpPolicyCheckTrialName))
    return true;
  return false;
}


bool OriginTrials::EventTimingEnabled(const ExecutionContext* executionContext) {
  if (RuntimeEnabledFeatures::EventTimingEnabledByRuntimeFlag())
    return true;

  const OriginTrialContext* context = OriginTrialContext::From(executionContext);
  if (!context) return false;
  if (context->IsTrialEnabled(kEventTimingTrialName))
    return true;
  return false;
}


bool OriginTrials::ExperimentalHardwareEchoCancellationEnabled(const ExecutionContext* executionContext) {
  if (RuntimeEnabledFeatures::ExperimentalHardwareEchoCancellationEnabledByRuntimeFlag())
    return true;

  const OriginTrialContext* context = OriginTrialContext::From(executionContext);
  if (!context) return false;
  if (context->IsTrialEnabled(kExperimentalHardwareEchoCancellationTrialName))
    return true;
  return false;
}


bool OriginTrials::FeaturePolicyJavaScriptInterfaceEnabled(const ExecutionContext* executionContext) {
  if (RuntimeEnabledFeatures::FeaturePolicyJavaScriptInterfaceEnabledByRuntimeFlag())
    return true;

  const OriginTrialContext* context = OriginTrialContext::From(executionContext);
  if (!context) return false;
  if (context->IsTrialEnabled(kFeaturePolicyJavaScriptInterfaceTrialName))
    return true;
  return false;
}


bool OriginTrials::InstalledAppEnabled(const ExecutionContext* executionContext) {
  if (RuntimeEnabledFeatures::InstalledAppEnabledByRuntimeFlag())
    return true;

  const OriginTrialContext* context = OriginTrialContext::From(executionContext);
  if (!context) return false;
  if (context->IsTrialEnabled(kInstalledAppTrialName))
    return true;
  return false;
}


bool OriginTrials::LowLatencyCanvasEnabled(const ExecutionContext* executionContext) {
  if (RuntimeEnabledFeatures::LowLatencyCanvasEnabledByRuntimeFlag())
    return true;

  const OriginTrialContext* context = OriginTrialContext::From(executionContext);
  if (!context) return false;
  if (context->IsTrialEnabled(kLowLatencyCanvasTrialName))
    return true;
  return false;
}


bool OriginTrials::MediaCapabilitiesEnabled(const ExecutionContext* executionContext) {
  if (RuntimeEnabledFeatures::MediaCapabilitiesEnabledByRuntimeFlag())
    return true;

  const OriginTrialContext* context = OriginTrialContext::From(executionContext);
  if (!context) return false;
  if (context->IsTrialEnabled(kMediaCapabilitiesTrialName))
    return true;
  return false;
}


bool OriginTrials::OriginTrialsSampleAPIEnabled(const ExecutionContext* executionContext) {
  if (RuntimeEnabledFeatures::OriginTrialsSampleAPIEnabledByRuntimeFlag())
    return true;

  const OriginTrialContext* context = OriginTrialContext::From(executionContext);
  if (!context) return false;
  if (context->IsTrialEnabled(kOriginTrialsSampleAPITrialName))
    return true;
  return false;
}


bool OriginTrials::OriginTrialsSampleAPIImpliedEnabled(const ExecutionContext* executionContext) {
  if (RuntimeEnabledFeatures::OriginTrialsSampleAPIImpliedEnabledByRuntimeFlag())
    return true;

  const OriginTrialContext* context = OriginTrialContext::From(executionContext);
  if (!context) return false;
  if (context->IsTrialEnabled(kOriginTrialsSampleAPIImpliedTrialName))
    return true;
  // TODO(crbug.com/831346): This check should be redundant with
  // GetImpliedTrials() being checked in OriginTrialContext::EnableTrialFromToken().
  // Remove when verified by further testing.
  if (context->IsTrialEnabled(kOriginTrialsSampleAPITrialName))
    return true;
  return false;
}


bool OriginTrials::RtcPeerConnectionIdEnabled(const ExecutionContext* executionContext) {
  if (RuntimeEnabledFeatures::RtcPeerConnectionIdEnabledByRuntimeFlag())
    return true;

  const OriginTrialContext* context = OriginTrialContext::From(executionContext);
  if (!context) return false;
  if (context->IsTrialEnabled(kRtcPeerConnectionIdTrialName))
    return true;
  return false;
}


bool OriginTrials::SignatureBasedIntegrityEnabled(const ExecutionContext* executionContext) {
  if (RuntimeEnabledFeatures::SignatureBasedIntegrityEnabledByRuntimeFlag())
    return true;

  const OriginTrialContext* context = OriginTrialContext::From(executionContext);
  if (!context) return false;
  if (context->IsTrialEnabled(kSignatureBasedIntegrityTrialName))
    return true;
  return false;
}


bool OriginTrials::StaleWhileRevalidateEnabled(const ExecutionContext* executionContext) {
  if (RuntimeEnabledFeatures::StaleWhileRevalidateEnabledByRuntimeFlag())
    return true;

  const OriginTrialContext* context = OriginTrialContext::From(executionContext);
  if (!context) return false;
  if (context->IsTrialEnabled(kStaleWhileRevalidateTrialName))
    return true;
  return false;
}


bool OriginTrials::TouchEventFeatureDetectionEnabled(const ExecutionContext* executionContext) {
  if (RuntimeEnabledFeatures::TouchEventFeatureDetectionEnabledByRuntimeFlag())
    return true;

  const OriginTrialContext* context = OriginTrialContext::From(executionContext);
  if (!context) return false;
  if (context->IsTrialEnabled(kTouchEventFeatureDetectionTrialName))
    return true;
  return false;
}


bool OriginTrials::WebXREnabled(const ExecutionContext* executionContext) {
  if (RuntimeEnabledFeatures::WebXREnabledByRuntimeFlag())
    return true;

  const OriginTrialContext* context = OriginTrialContext::From(executionContext);
  if (!context) return false;
  if (context->IsTrialEnabled(kWebXRTrialName))
    return true;
  return false;
}


bool OriginTrials::WebXRGamepadSupportEnabled(const ExecutionContext* executionContext) {
  if (RuntimeEnabledFeatures::WebXRGamepadSupportEnabledByRuntimeFlag())
    return true;

  const OriginTrialContext* context = OriginTrialContext::From(executionContext);
  if (!context) return false;
  if (context->IsTrialEnabled(kWebXRGamepadSupportTrialName))
    return true;
  return false;
}


Vector<AtomicString> OriginTrials::GetImpliedTrials(const String& trial_name) {
  if (trial_name == kOriginTrialsSampleAPITrialName) {
    Vector<AtomicString> implied_trials = {AtomicString(kOriginTrialsSampleAPIImpliedTrialName),};
    return implied_trials;
  }
  return Vector<AtomicString>();
}

} // namespace blink
